package ru.landgrafhomyak.db.serdha0.api.misc

import kotlin.jvm.JvmName
import ru.landgrafhomyak.db.serdha0.api.table.Table

/**
 * Descriptor of data column (of table or query). Used for schema manipulations and [access in queries builder][SelectedTable.selectColumn].
 *
 * @param RT Type in programming language to which database type is converted.
 * @param DT Descriptor of column type on database side.
 * @param TableUserExtension Type of [owner's table][Column.table] user expression for static reporting errors when this descriptor passed to wrong table.
 */
public interface Column<RT, DT : DatabaseType<RT>, TableUserExtension : Any> {
	/**
	 * Name of column for debugging, errors and raw schema access.
	 */
	@Suppress("INAPPLICABLE_JVM_NAME")
	@get:JvmName("name")
	public val name: String


	/**
	 * Descriptor of column type on database side.
	 */
	@Suppress("INAPPLICABLE_JVM_NAME")
	@get:JvmName("type")
	public val type: DT

	/**
	 * Table, that contains this column. For debugging.
	 */
	@Suppress("INAPPLICABLE_JVM_NAME")
	@get:JvmName("table")
	public val table: Table<TableUserExtension, *>
}