package ru.landgrafhomyak.db.serdha0.api.table

import ru.landgrafhomyak.db.serdha0.api.misc.Column
import ru.landgrafhomyak.db.serdha0.api.misc.DatabaseType


public interface TableUpgrade<tnUE : Any, toUE : Any> {
	public fun upgradeTable(context: Scope<tnUE, toUE>): tnUE

	public interface Scope<tnUE : Any, toUE : Any> : TableConstructor.Scope<tnUE> {
		public fun <RT, DT : DatabaseType<RT>> keepColumn(c: Column<RT, DT, toUE>): Column<RT, DT, tnUE>
		public fun <RT, DT : DatabaseType<RT>> keepColumnUntilUpgraded(c: Column<RT, DT, toUE>): Column<RT, DT, tnUE>
		public fun <RT, DT : DatabaseType<RT>> renameAndKeepColumn(c: Column<RT, DT, toUE>, newName: String): Column<RT, DT, tnUE>
		public fun deleteColumn(c: Column<*, *, toUE>)

		public fun keepIndex(i: Index<toUE>): Index<tnUE>
		public fun renameAndKeepIndex(i: Index<toUE>, newName: String): Index<tnUE>
		public fun deleteIndex(i: Index<toUE>)

		public fun keepCheck(i: CheckConstraint<toUE>): CheckConstraint<tnUE>
		public fun renameAndKeepCheck(i: CheckConstraint<toUE>, newName: String): CheckConstraint<tnUE>
		public fun deleteCheck(i: CheckConstraint<toUE>)

		public fun keepUniqueIndex(i: UniqueIndex<toUE>): UniqueIndex<tnUE>
		public fun renameAndKeepUniqueIndex(i: UniqueIndex<toUE>, newName: String): UniqueIndex<tnUE>
		public fun deleteUniqueIndex(i: UniqueIndex<toUE>)

		public fun <RT, DT : DatabaseType<RT>> keepDefaultValue(c: Column<RT, DT, tnUE>, d: ColumnDefaultValue<toUE, RT>): ColumnDefaultValue<tnUE, RT>
		public fun <RT, DT : DatabaseType<RT>> changeDefaultValue(c: Column<RT, DT, tnUE>, d: ColumnDefaultValue<toUE, RT>): ColumnDefaultValue<tnUE, RT>
		public fun deleteDefaultValue(i: ColumnDefaultValue<toUE, *>)

		public fun <ttUE : Any, RT, DT : DatabaseType<RT>> keepForeignRowReference(
			frr: ForeignRowReference<toUE, ttUE, RT, DT>,
			onDelete: ForeignRowReference.OnDelete? = null
		): ForeignRowReference<toUE, ttUE, RT, DT>

		public fun <ttoUE : Any, oRT, oDT : DatabaseType<oRT>, ttnUE : Any, nRT, nDT : DatabaseType<nRT>> keepForeignRowReferenceToUpdatedTable(
			frr: ForeignRowReference<toUE, ttoUE, oRT, oDT>,
			updatedType: nDT,
			updatedTable: Table<ttnUE, ttoUE>,
			onDelete: ForeignRowReference.OnDelete? = null
		): ForeignRowReference<tnUE, ttnUE, nRT, nDT>

		public fun <ttUE : Any, RT, DT : DatabaseType<RT>> renameAndKeepForeignRowReference(
			frr: ForeignRowReference<toUE, ttUE, RT, DT>,
			onDelete: ForeignRowReference.OnDelete? = null
		): ForeignRowReference<tnUE, ttUE, RT, DT>

		public fun <ttoUE : Any, oRT, oDT : DatabaseType<oRT>, ttnUE : Any, nRT, nDT : DatabaseType<nRT>> renameAndKeepForeignRowReferenceToUpdatedTable(
			frr: ForeignRowReference<toUE, ttoUE, oRT, oDT>,
			updatedType: nDT,
			updatedTable: Table<ttnUE, ttoUE>,
			onDelete: ForeignRowReference.OnDelete? = null
		): ForeignRowReference<tnUE, ttnUE, nRT, nDT>

		public fun deleteForeignRowReference(frr: ForeignRowReference<toUE, *, *, *>)

		public fun <ttUE : Any, RT, DT : DatabaseType<RT>> keepForeignRowReferenceUntilUpgraded(
			frr: ForeignRowReference<toUE, ttUE, RT, DT>,
			onDelete: ForeignRowReference.OnDelete? = null
		): ForeignRowReference<tnUE, ttUE, RT, DT>

		public fun <ttoUE : Any, oRT, oDT : DatabaseType<oRT>, ttnUE : Any, nRT, nDT : DatabaseType<nRT>> keepForeignRowReferenceToUpdatedTableUntilUpgraded(
			frr: ForeignRowReference<toUE, ttoUE, oRT, oDT>,
			updatedType: nDT,
			updatedTable: Table<ttnUE, ttoUE>,
			onDelete: ForeignRowReference.OnDelete? = null
		): ForeignRowReference<tnUE, ttnUE, nRT, nDT>
	}
}